#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Docs Team, all rights reserved

import unittest
from glxdocs.md2html import Md2Html
import os
import codecs


class TestDocs(unittest.TestCase):

    def setUp(self):
        self.md2html = Md2Html()
        self.cwd = os.path.dirname(os.path.abspath(__file__))
        self.md_file = os.path.join(self.cwd, 'README.md')
        self.html_file = os.path.join(self.cwd, 'README.html')

    def test_lang(self):
        self.assertEqual('en', self.md2html.lang)
        self.md2html.lang = None
        self.assertEqual('en', self.md2html.lang)

        self.assertRaises(TypeError, setattr, self.md2html, 'lang', 42)

    def test_charset(self):
        self.assertEqual('utf-8', self.md2html.charset)
        self.md2html.charset = None
        self.assertEqual('utf-8', self.md2html.charset)

        self.assertRaises(TypeError, setattr, self.md2html, 'charset', 42)

    def test_src_dirname(self):
        self.md2html.src_dirname = None
        self.assertIsNone(self.md2html.src_dirname)

        self.md2html.src_dirname = os.environ['HOME']
        self.assertEqual(os.environ['HOME'], self.md2html.src_dirname)
        self.md2html.src_dirname = None
        self.assertIsNone(self.md2html.src_dirname)

        self.assertRaises(TypeError, setattr, self.md2html, 'src_dirname', 42)
        self.assertRaises(FileNotFoundError, setattr, self.md2html, 'src_dirname', '42')
        self.assertRaises(NotADirectoryError, setattr, self.md2html, 'src_dirname', '/etc/hosts')

    def test_src_base_name(self):
        self.md2html.src_basename = None
        self.assertIsNone(self.md2html.src_basename)
        self.md2html.src_basename = 'Hello.42'
        self.assertEqual('Hello.42', self.md2html.src_basename)
        self.md2html.src_basename = None
        self.assertIsNone(self.md2html.src_basename)

        self.assertRaises(TypeError, setattr, self.md2html, 'src_basename', 42)

    def test_src_base_extension(self):
        self.md2html.src_extension = None
        self.assertIsNone(self.md2html.src_extension)
        self.md2html.src_extension = 'Hello.42'
        self.assertEqual('Hello.42', self.md2html.src_extension)
        self.md2html.src_extension = None
        self.assertIsNone(self.md2html.src_extension)

        self.assertRaises(TypeError, setattr, self.md2html, 'src_extension', 42)

    def test_dst_base_dir(self):
        self.assertIsNone(self.md2html.dst_dirname)
        self.md2html.dst_dirname = os.environ['HOME']
        self.assertEqual(os.environ['HOME'], self.md2html.dst_dirname)
        self.md2html.dst_dirname = None
        self.assertIsNone(self.md2html.dst_dirname)

        self.assertRaises(TypeError, setattr, self.md2html, 'dst_dirname', 42)
        self.assertRaises(FileNotFoundError, setattr, self.md2html, 'dst_dirname', '42')
        self.assertRaises(NotADirectoryError, setattr, self.md2html, 'dst_dirname', '/etc/hosts')

    def test_dst_base_name(self):
        self.assertIsNone(self.md2html.dst_basename)
        self.md2html.dst_basename = 'Hello.42'
        self.assertEqual('Hello.42', self.md2html.dst_basename)
        self.md2html.dst_basename = None
        self.assertIsNone(self.md2html.dst_basename)

        self.assertRaises(TypeError, setattr, self.md2html, 'dst_basename', 42)

    def test_dst_base_extension(self):
        self.assertIsNone(self.md2html.dst_extension)
        self.md2html.dst_extension = 'Hello.42'
        self.assertEqual('Hello.42', self.md2html.dst_extension)
        self.md2html.dst_extension = None
        self.assertIsNone(self.md2html.dst_extension)

        self.assertRaises(TypeError, setattr, self.md2html, 'dst_extension', 42)

    def test_input_file(self):
        self.md2html.input_file = codecs.open(self.md_file,
                                           mode="r",
                                           encoding="utf-8"
                                           )
        self.assertTrue(isinstance(self.md2html.input_file, codecs.StreamReaderWriter))
        self.md2html.input_file.close()

        self.assertRaises(TypeError, setattr, self.md2html, 'input_file', 42)

    def test_output_file(self):
        self.md2html.output_file = codecs.open(self.html_file,
                                            "w",
                                            encoding="utf-8",
                                            errors="xmlcharrefreplace"
                                            )

        self.assertTrue(isinstance(self.md2html.output_file, codecs.StreamReaderWriter))
        self.md2html.output_file.close()

        self.assertRaises(TypeError, setattr, self.md2html, 'output_file', 42)

        if os.path.exists(self.html_file):
            os.remove(self.html_file)

    def test_text(self):
        self.md2html.text = None
        self.assertIsNone(self.md2html.text)
        self.md2html.text = 'Hello.42'
        self.assertEqual('Hello.42', self.md2html.text)

        self.assertRaises(TypeError, setattr, self.md2html, 'text', 42)

    def test_body(self):
        self.md2html.body = None
        self.assertIsNone(self.md2html.body)
        self.md2html.body = 'Hello.42'
        self.assertEqual('Hello.42', self.md2html.body)

        self.assertRaises(TypeError, setattr, self.md2html, 'body', 42)

    def test_input_file_path(self):
        self.md2html.src_dirname = os.getcwd()
        self.md2html.src_basename = 'README'
        self.md2html.src_extension = '.md'
        self.assertEqual(
            os.path.join(os.getcwd(), '{0}{1}'.format('README', '.md')),
            self.md2html.input_file_path
        )
        self.md2html.src_dirname = None
        self.md2html.src_basename = None
        self.md2html.src_extension = None

    def test_output_file_path(self):
        self.md2html.dst_dirname = os.getcwd()
        self.md2html.dst_basename = 'README'
        self.md2html.dst_extension = '.html'
        self.assertEqual(
            os.path.join(os.getcwd(), '{0}{1}'.format('README', '.html')),
            self.md2html.output_file_path
        )
        self.md2html.dst_dirname = None
        self.md2html.dst_basename = None
        self.md2html.dst_extension = None

    def test_add_src_file_name(self):
        self.md2html.add_src_file_name(os.path.join(os.getcwd(), '{0}{1}'.format('README', '.md')))
        self.assertEqual(os.getcwd(), self.md2html.src_dirname)
        self.assertEqual('README', self.md2html.src_basename)
        self.assertEqual('.md', self.md2html.src_extension)

        self.md2html.add_src_file_name(None)
        self.assertIsNone(self.md2html.src_dirname)
        self.assertIsNone(self.md2html.src_basename)
        self.assertIsNone(self.md2html.src_extension)

        self.assertRaises(TypeError, self.md2html.add_src_file_name, 42)

    def test_add_dst_file_name(self):
        self.md2html.add_dst_file_name(os.path.join(os.getcwd(), '{0}{1}'.format('README', '.html')))
        self.assertEqual(os.getcwd(), self.md2html.dst_dirname)
        self.assertEqual('README', self.md2html.dst_basename)
        self.assertEqual('.html', self.md2html.dst_extension)

        self.md2html.add_dst_file_name(None)
        self.assertIsNone(self.md2html.dst_dirname)
        self.assertIsNone(self.md2html.dst_basename)
        self.assertIsNone(self.md2html.dst_extension)

        self.assertRaises(TypeError, self.md2html.add_dst_file_name, 42)

    def test_style(self):
        self.assertEqual(str, type(self.md2html.style))
        self.assertTrue(len(self.md2html.style) > 0)

    def test_html_code(self):
        self.md2html.body = ''
        prev_size = len(self.md2html.html_code)
        self.assertTrue(len(self.md2html.style) > 0)
        self.assertTrue(len(self.md2html.html_code) > 0)
        self.assertEqual(prev_size, len(self.md2html.html_code))
        self.md2html.body = '1'
        self.assertNotEqual(prev_size, len(self.md2html.html_code))
        self.md2html.body = ''
        self.assertEqual(prev_size, len(self.md2html.html_code))

    def test_run(self):
        self.md2html.add_src_file_name(self.md_file)
        self.md2html.add_dst_file_name(self.html_file)
        self.md2html.run()
        if os.path.exists(self.html_file):
            os.remove(self.html_file)


if __name__ == '__main__':
    unittest.main()



import contextlib
import os
import sys

from glxdocs.libs.directory import Directory


class File:
    def __init__(self):
        self.__directory = None
        self.__name = None
        self.__extension = None
        self.__overwrite = None
        self.__mode = None

        self.extension = None
        self.name = None
        self.directory = None
        self.path = None
        self.overwrite = None
        self.mode = None

    @property
    def extension(self):
        return self.__extension

    @extension.setter
    def extension(self, value=None):
        """
        Set the ``extension`` property value

        :param value: The extension of the file
        :type value: str or None
        :raise TypeError: When ``extension`` value is not a str type or None
        """
        if value and not isinstance(value, tuple) and not isinstance(value, str):
            raise TypeError('"extension" value must be a str type or None')
        if value is None:
            if self.extension is not None:
                self.__extension = None
        if value is not None and self.extension != value:
            self.__extension = value

    @property
    def name(self):
        return self.__name

    @name.setter
    def name(self, value=None):
        """
        Set the name property value

        :param value: The name of the file
        :type value: str or None
        :raise TypeError: When ``name`` value is not a str type or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"name" value must be a str type or None')
        if value is None:
            if self.name is not None:
                self.__name = None
        if value is not None and self.name != value:
            self.__name = value

    @property
    def directory(self):
        return self.__directory

    @directory.setter
    def directory(self, value):
        """
        Set the directory property value

        :param value: The path of the file
        :type value: str or None
        :raise TypeError: When ``directory`` value is not a str type or None
        """
        if value is None:
            value = Directory()

        if not isinstance(value, Directory):
            raise TypeError('"directory" value must be a Directory instance or None')

        if value != self.directory:
            self.__directory = value

    @property
    def path(self):
        if self.directory is not None and self.name is not None and self.name != "-":
            if self.extension:
                return os.path.join(
                    self.directory.path, "{0}{1}".format(self.name, self.extension)
                )
            else:
                return os.path.join(self.directory.path, self.name)
        else:
            return None

    @path.setter
    def path(self, value=None):
        """
        Set the path property value

        :param value: The path of the file
        :type value: str or None
        :raise TypeError: When value is not a str type or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"path" value must be a str type or None')

        self.extension = None
        self.name = None
        self.directory = None

        if value is not None:
            if value == "-":
                self.name = value
            else:
                # Filename isolation
                if os.path.basename(value).startswith("."):
                    self.name = ".{0}".format(os.path.basename(value)[1:].split(".")[0])
                else:
                    self.name = os.path.basename(value).split(".")[0]

                # Extension and post process
                spilt = os.path.basename(value).split(".")
                if spilt[0] == "":
                    spilt.pop(0)

                if len(spilt) >= 1:
                    spilt.pop(0)
                    if ".".join(spilt) != "":
                        self.extension = ".{0}".format(".".join(spilt))

                # Directory
                self.directory.path = os.path.dirname(value)

    @property
    def overwrite(self):
        return self.__overwrite

    @overwrite.setter
    def overwrite(self, value=None):
        """
        Set the overwrite property value

        :param value: If the file can be overwrite
        :type value: bool or None
        :raise TypeError: When value is not a bool type or None
        """
        if value is None:
            if self.overwrite is not False:
                self.__overwrite = False
        else:
            if not isinstance(value, bool):
                raise TypeError("'overwrite' value must be a bool type or None")
            if self.overwrite != value:
                self.__overwrite = value

    @property
    def mode(self):
        return self.__mode

    @mode.setter
    def mode(self, value=None):
        """
        Set the overwrite property value
        ========= ===============================================================
        Character Meaning
        --------- ---------------------------------------------------------------
        'r'       open for reading (default)
        'w'       open for writing, truncating the file first
        'x'       create a new file and open it for writing
        'a'       open for writing, appending to the end of the file if it exists
        'b'       binary mode
        't'       text mode (default)
        '+'       open a disk file for updating (reading and writing)
        'U'       universal newline mode (deprecated)
        ========= ===============================================================

        :param value: Mode use when open the file
        :type value: str or None
        :raise TypeError: When value is not a str type or None
        :raise ValueError: When value is in
        """
        if value is None:
            self.__mode = "rt"
        else:
            if not isinstance(value, str):
                raise TypeError("'mode' value must be a str type or None")
            for letter in value:
                if letter not in ["r", "w", "x", "a", "b", "t", "+", "U"]:
                    raise ValueError(
                        "'mode' value must be 'r', 'w', 'x', 'a', 'b', 't', '+', 'U' or a combination"
                    )
            if self.mode != value:
                self.__mode = value

    def is_binary(self):
        if os.path.exists(os.path.realpath(self.path)):
            if os.path.isfile(self.path) or os.path.islink(os.path.realpath(self.path)):
                fin = open(os.path.realpath(self.path), "rb")
                try:
                    chunk_size = 1024
                    while 1:
                        chunk = fin.read(chunk_size)
                        if b"\0" in chunk:
                            return True
                        if len(chunk) < chunk_size:
                            break
                finally:
                    fin.close()

                return False

        else:
            raise FileNotFoundError("File Not Found")

    def is_text(self):
        if os.path.exists(os.path.realpath(self.path)):
            if os.path.isfile(self.path) or os.path.islink(os.path.realpath(self.path)):
                fin = open(os.path.realpath(self.path), "rb")
                try:
                    chunk_size = 1024
                    while 1:
                        chunk = fin.read(chunk_size)
                        if b"\0" in chunk:
                            return False
                        if len(chunk) < chunk_size:
                            break
                finally:
                    fin.close()

                return True

        else:
            raise FileNotFoundError("File Not Found")

    def found_best_output_file_name(self):
        output_file = None
        if os.path.exists(self.path):
            if self.overwrite:
                output_file = self.path
            else:
                i = 1
                if self.extension:
                    extension = self.extension
                else:
                    extension = ""
                while os.path.exists(
                    os.path.join(
                        self.directory.path,
                        "{0}-{1}{2}".format(self.name, i, extension),
                    )
                ):
                    i += 1
                output_file = os.path.join(
                    self.directory.path, "{0}-{1}{2}".format(self.name, i, extension)
                )

        return output_file

    @contextlib.contextmanager
    def smart_open(self):
        if self.name == "-":
            if self.mode is None or self.mode == "" or "r" in self.mode:
                fh = sys.stdin
            else:
                fh = sys.stdout
        else:
            fh = open(file=self.path, mode=self.mode)

        try:
            yield fh
        finally:
            if self.name != "-":
                fh.close()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import sys
from unittest import TestCase, main
from glxdocs.yaml2md import Yaml2Md
from glxdocs.yaml2md import State

# It script it publish under GNU GENERAL PUBLIC LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Docs Team, all rights reserved


class TestDocs(TestCase):

    def setUp(self):
        self.yaml2md = Yaml2Md()
        self.cwd = os.path.dirname(os.path.abspath(__file__))
        self.roles_src_path = os.path.join(self.cwd, 'data', 'roles')
        self.roles_dst_reference = os.path.join(self.cwd, 'data', 'reference', 'role')

    def test_yamle2md_property_state(self):
        self.yaml2md.state = State.YAML
        self.assertEqual(State.YAML, self.yaml2md.state)

        self.yaml2md.state = None
        self.assertEqual(State.TEXT, self.yaml2md.state)

        self.assertRaises(TypeError, setattr, self.yaml2md, "state", "Hello42")

    def test_yaml2md(self):

        ignore_role_list = [
            'galaxie_clans.dns',
            'galaxie_clans.clan_host',
            'galaxie_clans.system_base',
            'galaxie_clans.system_users'
        ]

        for element in os.listdir(self.roles_src_path):
            default_directory = os.path.abspath(os.path.join(self.roles_src_path, element, 'defaults'))
            if not os.path.isdir(default_directory) or element in ignore_role_list:
                continue
            for path, sub_dirs, files in os.walk(default_directory):
                for filename in files:
                    if filename.startswith("."):
                        continue

                    self.yaml2md.file_input.path = os.path.join(path, filename)
                    self.yaml2md.file_output.path = self.roles_dst_reference + "_" + element + ".md"

                    self.yaml2md.convert_file()

                    sys.stdout.write("Converted {0}".format(self.roles_dst_reference + "_" + element + "\n"))
                    sys.stdout.flush()


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

# It script it publish under GNU GENERAL PUBLIC LICENSE
# http://www.gnu.org/licenses/gpl-3.0.en.html
# Author: the Galaxie Docs Team, all rights reserved

import codecs
import os
from argparse import ArgumentParser

import markdown


# Inspired from: https://chriswarrick.com/blog/2014/09/15/python-apps-the-right-way-entry_points-and-scripts/
class Md2Html:
    def __init__(self):
        self.__lang = None
        self.__charset = None
        self.__src_base_dir = None
        self.__src_base_name = None
        self.__src_base_extension = None

        self.__dst_base_dir = None
        self.__dst_base_name = None
        self.__dst_base_extension = None

        self.__input_file = None
        self.__output_file = None
        self.__text = None
        self.__html = None

        self.lang = "en"
        self.charset = "utf-8"

        self.src_dirname = None
        self.src_basename = None
        self.src_extension = None

        self.dst_dirname = None
        self.dst_basename = None
        self.dst_extension = None

        self.input_file = None
        self.output_file = None
        self.text = None
        self.body = None

    @property
    def lang(self):
        """
        A reference to HTTP_ACCEPT_LANGUAGE

        :return: Reference List HTTP_ACCEPT_LANGUAGE
        :rtype: str
        """
        return self.__lang

    @lang.setter
    def lang(self, value=None):
        """
        Set the ``lang`` property value

        Default value is ``en`` and be restore when ``lang`` is set to None

        :param value: The lang use on the html document
        :type value: str or none
        :raise TypeError: When ``lang`` value is not str type or None
        """
        if value is None:
            value = "en"
        if value and not isinstance(value, str):
            raise TypeError('"lang" value must be a str type or None')
        if self.lang != value:
            self.__lang = value

    @property
    def charset(self):
        """
        The ``charset`` property, Specify the character encoding for the HTML document

        http://webcheatsheet.com/html/character_sets_list.php

        :return: the character encoding
        :rtype: str
        """
        return self.__charset

    @charset.setter
    def charset(self, value=None):
        """
        set the ``charset`` property

        Default value is 'utf-8' and be restore when ``charset`` is set to None

        In theory, any character encoding can be used, but no browser understands all of them.
        The more widely a character encoding is used, the better the chance that a browser will understand it.

        https://www.iana.org/assignments/character-sets/character-sets.txt

        :param value: the charset use on html and markdown document
        :type value: str or none
        :raise TypeError: ``charset`` is not a str type or None
        """
        if value is None:
            value = "utf-8"
        if value and not isinstance(value, str):
            raise TypeError('"charset" value must be a str type or None')
        if self.charset != value:
            self.__charset = value

    @property
    def src_dirname(self):
        """
        The ``src_dirname`` property is use to store source directory path where is locate the input file

        :return: The source directory
        :rtype: str or None
        """
        return self.__src_base_dir

    @src_dirname.setter
    def src_dirname(self, value=None):
        """
        Set the ``src_dirname`` property value

        :param value: The location path of the source directory
        :type value: str or None
        :raise TypeError: ``src_dirname`` value is not a str type or None
        :raise FileNotFoundError: ``src_dirname`` value is path it not exist or None
        :raise NotADirectoryError: ``src_dirname`` value is path it not a directory or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"src_base_dir" value must be a str type or None')
        if value and not os.path.exists(os.path.realpath(value)):
            raise FileNotFoundError('"src_base_dir" path do not exist')
        if value and not os.path.isdir(os.path.realpath(value)):
            raise NotADirectoryError('"src_base_dir" is not a directory')

        if self.src_dirname != value:
            self.__src_base_dir = value

    @property
    def src_basename(self):
        """
        The ``src_basename`` property is the the name of a source file without extension and without directory path

        :return: The source file basename
        :rtype: str or None
        """
        return self.__src_base_name

    @src_basename.setter
    def src_basename(self, value=None):
        """
        Set the ``src_basename`` property

        :param value: The source file basename
        :type value: str or None
        :raise TypeError: ``src_basename`` value is not a str type or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"src_base_name" value must be a str type or None')
        if self.src_basename != value:
            self.__src_base_name = value

    @property
    def src_extension(self):
        """
        The ``src_extension`` property is the the suffix part

        Example: ``Hello.42`` have ``.42`` as extension

        :return: The source file extension
        :rtype: str or None
        """
        return self.__src_base_extension

    @src_extension.setter
    def src_extension(self, value=None):
        """
        Set ``src_extension`` property value.

        :param value: the extension of the source file
        :type value: str or None
        :raise TypeError: When ``src_extension`` value is not a str type or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"src_base_extension" value must be a str type or None')
        if self.src_extension != value:
            self.__src_base_extension = value

    @property
    def dst_dirname(self):
        """
        The ``dst_dirname`` property is use to store destination directory path where is locate the output file

        :return: The destination directory
        :rtype: str or None
        """
        return self.__dst_base_dir

    @dst_dirname.setter
    def dst_dirname(self, value=None):
        """
        Set the ``dst_dirname`` property value

        :param value: The location path of the destination directory
        :type value: str or None
        :raise TypeError: When ``dst_dirname`` value is not a str type or None
        :raise FileNotFoundError: When ``dst_dirname`` value is path it not exist or None
        :raise NotADirectoryError: When ``dst_dirname`` value is path it not a directory or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"dst_base_dir" value must be a str type or None')
        if value and not os.path.exists(os.path.realpath(value)):
            raise FileNotFoundError('"dst_base_dir" path do not exist')
        if value and not os.path.isdir(os.path.realpath(value)):
            raise NotADirectoryError('"dst_base_dir" is not a directory')
        if self.dst_dirname != value:
            self.__dst_base_dir = value

    @property
    def dst_basename(self):
        """
        The ``dst_basename`` property is the the name of a destination file without extension and without directory path

        :return: The destination file basename
        :rtype: str or None
        """
        return self.__dst_base_name

    @dst_basename.setter
    def dst_basename(self, value=None):
        """
        Set the ``dst_basename`` property

        :param value: The destination file basename
        :type value: str or None
        :raise TypeError: When ``dst_basename`` value is not a str type or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"dst_base_name" value must be a str type or None')
        if self.dst_basename != value:
            self.__dst_base_name = value

    @property
    def dst_extension(self):
        """
        The ``dst_extension`` property is the the suffix part

        Example: ``Hello.42`` have ``.42`` as extension

        :return: The destination file extension
        :rtype: str or None
        """
        return self.__dst_base_extension

    @dst_extension.setter
    def dst_extension(self, value=None):
        """
        Set ``dst_extension`` property value.

        :param value: the extension of the destination file
        :type value: str or None
        :raise TypeError: When ``dst_extension`` value is not a str type or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"dst_base_extension" value must be a str type or None')
        if self.dst_extension != value:
            self.__dst_base_extension = value

    @property
    def input_file(self):
        """
        The ``input_file`` property store the codecs.StreamReaderWriter object of the input file.

        :return: The input file descriptor
        :rtype: codecs.StreamReaderWriter or None
        """
        return self.__input_file

    @input_file.setter
    def input_file(self, value=None):
        """
        Set ``input_file`` property value

        :param value: The input file descriptor
        :type value: codecs.StreamReaderWriter or None
        :raise TypeError: When ``input_file`` value is not a codecs.StreamReaderWriter type or None
        """
        if value and not isinstance(value, codecs.StreamReaderWriter):
            raise TypeError(
                '"input_file" value must be a codecs.StreamReaderWriter type or None'
            )
        if self.input_file != value:
            self.__input_file = value

    @property
    def output_file(self):
        """
        The ``output_file`` property store the codecs.StreamReaderWriter object of the output file.

        :return: The output file descriptor
        :rtype: codecs.StreamReaderWriter
        """
        return self.__output_file

    @output_file.setter
    def output_file(self, value=None):
        """
        Set ``output_file`` property value

        :param value: The output file descriptor
        :type value: codecs.StreamReaderWriter or None
        :raise TypeError: When ``output_file`` value is not a codecs.StreamReaderWriter type or None
        """
        if value and not isinstance(value, codecs.StreamReaderWriter):
            raise TypeError(
                '"output_file" value must be a codecs.StreamReaderWriter type or None'
            )
        if self.output_file != value:
            self.__output_file = value

    @property
    def text(self):
        """
        The ``text`` property store the text imported from input file

        :return: input file imported text
        :rtype: str or None
        """
        return self.__text

    @text.setter
    def text(self, value=None):
        """
        Set the ``text`` property

        :param value: input file imported text
        :type value: str or None
        :raise TypeError: When ``text`` value is not a str type or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"text" value must be a str type or None')
        if self.text != value:
            self.__text = value

    @property
    def body(self):
        """
        The ``body`` property store the html body part generate by MarkDown python module.

        :return: The html body part
        :rtype: str or None
        """
        return self.__html

    @body.setter
    def body(self, value=None):
        """
        Set the ``body`` property

        :param value: The html body part
        :type value: str or None
        :raise TypeError: When ``body`` value is not a str type or None
        """
        if value and not isinstance(value, str):
            raise TypeError('"text" value must be a str type or None')
        if self.body != value:
            self.__html = value

    @property
    def input_file_path(self):
        """
        The ``input_file_path`` is a read only property is return the real path of the source file.

        :return: The real path of the source file
        :rtype: str
        """
        return os.path.join(
            self.src_dirname, f"{self.src_basename}{self.src_extension}"
        )

    @property
    def output_file_path(self):
        """
        The ``output_file_path`` is a read only property is return the real path of the destination file.

        :return: The real path of the destination file
        :rtype: str
        """
        return os.path.join(
            self.dst_dirname, f"{self.dst_basename}{self.dst_extension}"
        )

    def add_src_file_name(self, value=None):
        """
        It function assist it import the source file path, basically it set
        ``src_dirname``, ``src_basename`` and ``src_extension`` automatically from a file path

        :param value: the input file path
        :type value: str or None
        :raise TypeError: When ``value`` is not a str type or None
        """
        if value is None:
            self.src_dirname = None
            self.src_basename = None
            self.src_extension = None
            return
        if not isinstance(value, str):
            raise TypeError('"value" must be a str type or None')
        self.src_dirname = os.path.dirname(value)
        self.src_basename = os.path.splitext(os.path.basename(value))[0]
        self.src_extension = os.path.splitext(os.path.basename(value))[1]

    def add_dst_file_name(self, value):
        """
        It function assist it import the destination file path, basically it set
        ``dst_dirname``, ``dst_basename`` and ``dst_extension`` automatically from a file path

        :param value: the output file path
        :type value: str or None
        :raise TypeError: When ``value`` is not a str type or None
        """
        if value is None:
            self.dst_dirname = None
            self.dst_basename = None
            self.dst_extension = None
            return
        if not isinstance(value, str):
            raise TypeError('"value" must be a str type or None')
        self.dst_dirname = os.path.dirname(value)
        self.dst_basename = os.path.splitext(os.path.basename(value))[0]
        self.dst_extension = os.path.splitext(os.path.basename(value))[1]

    @property
    def style(self):
        """
        The ``style`` is a read only property it return the style part of the html code

        :return: The style part of the html code
        :rtype: list
        """
        css = [
            "body { font-family: Impact, Charcoal, sans-serif;}",
            ".hll { background-color: #ffffcc }",
            ".c { color: #408080; font-style: italic } /* Comment */",
            ".err { border: 1px solid #FF0000 } /* Error */",
            ".k { color: #008000; font-weight: bold } /* Keyword */",
            ".o { color: #666666 } /* Operator */",
            ".cm { color: #408080; font-style: italic } /* Comment.Multiline */",
            ".cp { color: #BC7A00 } /* Comment.Preproc */",
            ".c1 { color: #408080; font-style: italic } /* Comment.Single */",
            ".cs { color: #408080; font-style: italic } /* Comment.Special */",
            ".gd { color: #A00000 } /* Generic.Deleted */",
            ".ge { font-style: italic } /* Generic.Emph */",
            ".gr { color: #FF0000 } /* Generic.Error */",
            ".gh { color: #000080; font-weight: bold } /* Generic.Heading */",
            ".gi { color: #00A000 } /* Generic.Inserted */",
            ".go { color: #888888 } /* Generic.Output */",
            ".gp { color: #000080; font-weight: bold } /* Generic.Prompt */",
            ".gs { font-weight: bold } /* Generic.Strong */",
            ".gu { color: #800080; font-weight: bold } /* Generic.Subheading */",
            ".gt { color: #0044DD } /* Generic.Traceback */",
            ".kc { color: #008000; font-weight: bold } /* Keyword.Constant */",
            ".kd { color: #008000; font-weight: bold } /* Keyword.Declaration */",
            ".kn { color: #008000; font-weight: bold } /* Keyword.Namespace */",
            ".kp { color: #008000 } /* Keyword.Pseudo */",
            ".kr { color: #008000; font-weight: bold } /* Keyword.Reserved */",
            ".kt { color: #B00040 } /* Keyword.Type */",
            ".m { color: #666666 } /* Literal.Number */",
            ".s { color: #BA2121 } /* Literal.String */",
            ".na { color: #7D9029 } /* Name.Attribute */",
            ".nb { color: #008000 } /* Name.Builtin */",
            ".nc { color: #0000FF; font-weight: bold } /* Name.Class */",
            ".no { color: #880000 } /* Name.Constant */",
            ".nd { color: #AA22FF } /* Name.Decorator */",
            ".ni { color: #999999; font-weight: bold } /* Name.Entity */",
            ".ne { color: #D2413A; font-weight: bold } /* Name.Exception */",
            ".nf { color: #0000FF } /* Name.Function */",
            ".nl { color: #A0A000 } /* Name.Label */",
            ".nn { color: #0000FF; font-weight: bold } /* Name.Namespace */",
            ".nt { color: #008000; font-weight: bold } /* Name.Tag */",
            ".nv { color: #19177C } /* Name.Variable */",
            ".ow { color: #AA22FF; font-weight: bold } /* Operator.Word */",
            ".w { color: #bbbbbb } /* Text.Whitespace */",
            ".mf { color: #666666 } /* Literal.Number.Float */",
            ".mh { color: #666666 } /* Literal.Number.Hex */",
            ".mi { color: #666666 } /* Literal.Number.Integer */",
            ".mo { color: #666666 } /* Literal.Number.Oct */",
            ".sb { color: #BA2121 } /* Literal.String.Backtick */",
            ".sc { color: #BA2121 } /* Literal.String.Char */",
            ".sd { color: #BA2121; font-style: italic } /* Literal.String.Doc */",
            ".s2 { color: #BA2121 } /* Literal.String.Double */",
            ".se { color: #BB6622; font-weight: bold } /* Literal.String.Escape */",
            ".sh { color: #BA2121 } /* Literal.String.Heredoc */",
            ".si { color: #BB6688; font-weight: bold } /* Literal.String.Interpol */",
            ".sx { color: #008000 } /* Literal.String.Other */",
            ".sr { color: #BB6688 } /* Literal.String.Regex */",
            ".s1 { color: #BA2121 } /* Literal.String.Single */",
            ".ss { color: #19177C } /* Literal.String.Symbol */",
            ".bp { color: #008000 } /* Name.Builtin.Pseudo */",
            ".vc { color: #19177C } /* Name.Variable.Class */",
            ".vg { color: #19177C } /* Name.Variable.Global */",
            ".vi { color: #19177C } /* Name.Variable.Instance */",
            ".il { color: #666666 } /* Literal.Number.Integer.Long */",
        ]

        to_return = ""
        for line in css:
            to_return = to_return + line + "\n"

        return to_return

    @property
    def html_code(self):
        """
        The ``html_code`` is a a read only property ir return the html code with everything.

        Basically it merge the result of ``body``, ``style``, ``charset`` and ``lang``.

        :return: html code
        :rtype: str
        """
        # return (
        #     "<!DOCTYPE html>\n"
        #     '<html lang="{4}">\n'
        #     "<head>\n"
        #     '<meta charset="{3}">\n'
        #     "<style>\n"
        #     "{2}\n"
        #     "</style>\n"
        #     "</head>\n"
        #     "<body>\n"
        #     "{0}\n"
        #     "</body>\n"
        #     "</html>\n"
        #     "".format(self.body, "}", self.style, self.charset, self.lang)
        # )

        return f"""<!DOCTYPE html>
<html lang="{self.lang}">
    <head>
        <meta charset="{self.charset}">
        <style>
            {self.style}
        </style>
    </head>
    <body>
        {self.body}
    </body>
</html>
"""
    def run(self):
        """
        That function , make the job.

        Everything must be set before star it function.

        Take a look to add_src_file_name and add_dst_file_name
        """
        with codecs.open(
                self.input_file_path,
                mode="r",
                encoding=self.charset
        ) as self.input_file, codecs.open(
            self.output_file_path,
            "w",
            encoding=self.charset,
            errors="xmlcharrefreplace",
        ) as self.output_file:
            self.text = self.input_file.read()

            self.body = markdown.markdown(
                self.text,
                output_format="html5",
                extensions=[
                    "extra",
                    "abbr",
                    "attr_list",
                    "def_list",
                    "fenced_code",
                    "footnotes",
                    "md_in_html",
                    "tables",
                    "admonition",
                    "codehilite",
                    "legacy_attrs",
                    "legacy_em",
                    "meta",
                    # 'nl2br',
                    "sane_lists",
                    "smarty",
                    "toc",
                    "wikilinks",
                ],
            )

            self.output_file.write(self.html_code)


def main():  # pragma: no cover
    md2html = Md2Html()
    parser = ArgumentParser(
        description="Galaxie Docs - Markdown to HTML", epilog="Developed under GPLv3+ license"
    )
    parser.add_argument("source", help="source file path")
    parser.add_argument("destination", help="destination file path")
    parser.add_argument(
        "--lang", dest="lang", default="en", help="HTTP_ACCEPT_LANGUAGE"
    )

    parser.add_argument(
        "--charset", dest="charset", default="utf-8", help="character sets"
    )

    args = parser.parse_args()

    md2html.lang = args.lang
    md2html.charset = args.charset
    if os.sep in args.source:
        md2html.add_src_file_name(args.source.split(os.sep, 1)[1])
    else:
        md2html.add_src_file_name(args.source)

    md2html.add_dst_file_name(os.path.abspath(args.destination))
    md2html.run()



import os


class Directory:
    def __init__(self):
        self.__path = None
        self.path = None

    @property
    def path(self):
        return self.__path

    @path.setter
    def path(self, value):
        if value is None:
            value = os.getcwd()

        if not isinstance(value, str):
            raise TypeError("path property value must be a str type or None")

        if self.path != value:
            self.__path = value



"""
yaml2md - Galaxie YAML to MarkDown utility
"""

from argparse import ArgumentParser
from enum import Enum
from re import sub

from glxdocs.libs.file import File


class State(Enum):
    """
    State class, consist of an ``Enum`` class heritage and a set of constants

    where: TEXT=0 and YAML=1
    """
    TEXT = 0
    YAML = 1


class Yaml2Md:
    """
    Yaml2Md class
    """

    def __init__(self):
        self.file_input = File()
        self.file_output = File()
        self.file_output.mode = "w"
        self.strip_regex = r"\s*(:?\[{3}|\]{3})\d?$"
        self.__state = None
        self.state = None

    @property
    def state(self):
        return self.__state

    @state.setter
    def state(self, value: State):
        if value is None:
            value = State.TEXT

        if value and not isinstance(value, State):
            raise TypeError("state property 'value' must be int type or None")
        if self.state != value:
            self.__state = value

    def convert_lines(self, lines):
        """
        That function is call by ``convert_file`` function, the hard work is done by it function:

        The ``lines`` argv will be estimated inside a loop, the text will be clean, and commentary estimated for get
        the Markdown information.

        :param lines: A lines list to convert in to Yaml
        :type lines: list
        :return: Converted lines iterator
        :rtype: str
        """
        self.state = State.TEXT
        last_text_line = ""
        for line in lines:
            # Remove any white spaces at the end of the string
            line = line.rstrip()

            # Cleaning
            if not line:
                # do not change state if the line is empty
                pass
            elif line == "---":
                # ignore yaml tags
                pass

            # Yaml or not Yaml what is the question
            elif line.startswith("# ") or line == "#":
                if self.state != State.TEXT:
                    yield "```"
                    yield ""

                line = last_text_line = sub(self.strip_regex, "", line)[2:]
                self.state = State.TEXT
                yield line
            else:
                if self.state != State.YAML:
                    if not last_text_line.endswith("```"):
                        yield ""
                        yield "``` yaml"

                line = sub(self.strip_regex, "", line)
                yield line
                self.state = State.YAML

        # If last line was a YAML section open
        if self.state == State.YAML:
            if not last_text_line.endswith("```"):
                yield "```"

    def convert_file(self) -> None:
        """
        After have set ``file_input.path`` ``file_output.path`` properties value, that function can be call.

        The function open input file as read only and output file with write permission.
        For each line inside input file a conversion is done and write in to output file.

        Note: If the output file do not exist it will be created, but the parent directory must exist.
        """
        with self.file_input.smart_open() as input_file, self.file_output.smart_open() as output_file:
            for line in self.convert_lines(input_file.readlines()):
                output_file.write(line.rstrip())
                output_file.write("\n")
                output_file.flush()


def main():  # pragma: no cover
    """
    The entry point of ``yaml2md``, it is use by setup-tools as console script.
    Tha function define the CLI parser and the Yaml2Md class object
    """
    parser = ArgumentParser(
        description="Galaxie Docs - YAML to Markdown", epilog="Developed under GPLv3+ license"
    )
    parser.add_argument(
        "source_file", metavar="source_file",
        nargs='?',
        help="A pathname of an YAML input file. "
             "If no file operands are specified, the standard input shall be used. "
             "If a file is '-', the utility read from the standard input at that point in the sequence."
    )
    parser.add_argument(
        "target_file", metavar="target_file",
        nargs='?',
        help="A pathname of an target file. "
             "If no file operands are specified, the standard output shall be used. "
             "If a file is '-', the utility write to the standard output at that point in the sequence."
    )
    parser.add_argument(
        "--strip-regex",
        metavar="strip_regex",
        default=r"\s*(:?\[{3}|\]{3})\d?$",
        help=(
            "Regex which will remove everything it matches. "
            "Can be used e.g. to remove fold markers from headings. "
            "Example to strip out [[[,]]] fold markers use: "
            r"'\s*(:?\[{3}|\]{3})\d?$'. "
            "Check the README for more details."
        ),
    )

    args = parser.parse_args()

    yaml2md = Yaml2Md()
    yaml2md.file_input.path = args.source_file
    yaml2md.file_output.path = args.target_file
    yaml2md.strip_regex = rf"{args.strip_regex}"

    yaml2md.convert_file()
